// voice album project
//
#include <UD.h>
#include <SPI.h>
//Here is an example of audioZero https://www.arduino.cc/en/Tutorial/SimpleAudioPlayerZero
#include <AudioZero.h>

// D12口低电平将关闭PAM8403音频功放
// When D12 is low level will turn off pam8403 audio amplifier
const byte nShoutdown = 12;

// D32 pin for SPI Flash CS pin
const byte flashCS = 32;

// 5个按钮的编号对应到Firebeetle M0 Pin
// The numbers of the five button correspond to firebeetle M0 pin
// Button  number      :| 0 | 1 | 2 | 3 | 4 |
// Firebeetle M0    Pin:| D0| D1| D3| D5| D6|

// 记录按钮按下的时刻
// Record the time when the button is pressed
volatile unsigned long triggertime = 0; 
// 去抖期的时长 （毫秒）
// Duration of dithering (mS)
volatile unsigned long delta = 500;   
volatile boolean buttonStatus[5] = {0};
volatile byte soundNumber = 0;
volatile boolean startPlay = 0;
String title = String(".wav");

// 播放控制
// Playback control
void controlPlay(void) {
  volatile byte i = 0;

  // 如果有按钮按下将停止音频播放
  // If a button is pressed, the audio will stop playing
  AudioZero.stop();
  Serial.println("stop ");
  // 如果没有播放过这个音频，此时就播放按钮对应的这个音频
  // If the audio has not been played, the audio corresponding to the button will be played at this time
  if (0 == buttonStatus[soundNumber]) {
    title = String(".wav");
    title =  soundNumber + title;
    startPlay = 1;
    for (i = 0; i < 5; i++) {
      buttonStatus[i] = 0;
    }
    buttonStatus[soundNumber] = 1;
  }
  else {
    buttonStatus[soundNumber] = 0;
  }

}

// 读取5个按钮的值，按钮的编号对应到Firebeetle M0 Pin如下
// Read the values of the five buttons. The button numbers correspond to firebeetle M0 pin as follows
// Button  number      :| 0 | 1 | 2 | 3 | 4 |
// Firebeetle M0    Pin:| D0| D1| D3| D5| D6|
void readButton(void) {

  // 如果第二次按钮中断间隔时间大于消除抖动时间，说明是一次正常的按键动作
  // If the interval between the second button interruption is greater than the time to eliminate jitter, it is a normal key action
  if (millis() - triggertime > delta) {
    triggertime = millis();
    // 读取D0引脚的电平，如果是按钮0按下调用控制播放功能
    // Read the level of D0 pin, if it is the button 0, press it to call the control playback function
    if (digitalRead(0) == 0) {
      soundNumber = 0;
      controlPlay();
      return;
    }

    // 读取D1引脚的电平，如果是按钮1按下调用控制播放功能
    // Read the level of D1 pin, if it is the button 1, press it to call the control playback function
    if (digitalRead(1) == 0) {
      soundNumber = 1;
      controlPlay();
      return;
    }

    // 读取D3引脚的电平，如果是按钮2按下调用控制播放功能
    // Read the level of D3 pin, if it is the button 2, press it to call the control playback function
    if (digitalRead(3) == 0) {
      soundNumber = 2;
      controlPlay();
      return;
    }

    // 读取D5引脚的电平，如果是按钮3按下调用控制播放功能
    // Read the level of D5 pin, if it is the button 3, press it to call the control playback function
    if (digitalRead(5) == 0) {
      soundNumber = 3;
      controlPlay();
      return;
    }

    // 读取D6引脚的电平，如果是按钮4按下调用控制播放功能
    // Read the level of D6 pin, if it is the button 4, press it to call the control playback function
    if (digitalRead(6) == 0) {
      soundNumber = 4;
      controlPlay();
      return;
    }
  }
}

// 初始化设置函数，程序从这里开始
// Initialization setting function, the program starts from here
void setup()
{
  // 设置音频功放关闭控制脚为输出
  // Set the audio amplifier off control pin as output
  pinMode(nShoutdown, OUTPUT);
  // 关闭音频功放
  // Audio power amplifier off
  digitalWrite(nShoutdown, LOW);

  // 设置5个按钮接口为输入，并且上拉使能，按钮的编号对应到Firebeetle M0 Pin如下
  // The numbers of the five button correspond to firebeetle M0 pin
  // Button  number      :| 0 | 1 | 2 | 3 | 4 |
  // Firebeetle M0    Pin:| D0| D1| D3| D5| D6|
  // 设置| D0| D1| D3| D5| D6|为输入中断
  // Set | D0 | D1 | D3 | D5 | D6 | as input interrupt
  pinMode(0, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(0), readButton, FALLING );
  pinMode(1, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(1), readButton, FALLING );
  pinMode(3, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(3), readButton, FALLING );
  pinMode(5, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(5), readButton, FALLING );
  pinMode(6, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(6), readButton, FALLING );

  Serial.begin(115200);
  Serial.print("Initializing SD card...");
  //D32 pin for SPI Flash CS pin
  if (!UD.begin(flashCS)) {
    Serial.println(" failed!");
    while (true);
  }
  Serial.println(" done.");
  // 44100kHz mono sample rate
  AudioZero.begin(44100);
}

// 程序主循环
// Program main loop
void loop ()
{
  byte flag = 0;
  // 如果startPlay标志置位，控制播放
  // If the startplay flag is set, the playback is controlled
  if (1 == startPlay) {
    startPlay = 0;
    // 按照音频文件名读取文件
    // Read audio file name according to
    UDFile myFile = UD.open(title);
    if (!myFile) {
      // if the file didn't open, print an error and stop
      Serial.println("error opening .wav file");
    }
    else {
      // 开启音频功放
      // Turn on audio amplifier
      digitalWrite(nShoutdown, HIGH);
      Serial.print("Playing ");
      Serial.println(soundNumber);
      // 返回byte: false:音频播放完毕; true:音频播放过程中中断停止
      // Return byte: false: audio playback finished; true: interrupt stop during audio playback
      // until the file is not finished
      flag = AudioZero.play(myFile);
      if (false == flag) {
        buttonStatus[soundNumber] = 0;
      }
      Serial.println("End of file. Thank you for listening!");

      // 播放完毕关闭音频功放
      // Turn off the audio amplifier after playing
      digitalWrite(nShoutdown, LOW);
    }
  }
}
